@@ -2,16 +2,24 @@
 
 import argparse
 
-import meshcut
 import numpy as np
-import stl
 from OpenGL import GL
 
-from ..calc import calc_distance  # pylint: disable=E0402
+from ..ext import stl
+from ..ext.meshcut import meshcut
 from ..input_plugins_base import DrawReaderBase
 
 
 class DrawReader(DrawReaderBase):
+    @staticmethod
+    def arg_parser(parser) -> None:
+        parser.add_argument(
+            "--stlread-zslice",
+            help="stlread: slice at postion z (stl)",
+            type=str,
+            default=None,
+        )
+
     def __init__(self, filename: str, args: argparse.Namespace = None):
         """slicing and converting stl into single segments."""
         self.filename = filename
@@ -33,12 +41,12 @@ class DrawReader(DrawReaderBase):
         print(f"STL: INFO: z_min={min_z}, z_max={max_z}")
 
         slice_z = None
-        if args.zslice:  # type: ignore
-            if args.zslice.endswith("%"):  # type: ignore
-                percent = float(args.zslice[:-1])  # type: ignore
+        if args.stlread_zslice:  # type: ignore
+            if args.stlread_zslice.endswith("%"):  # type: ignore
+                percent = float(args.stlread_zslice[:-1])  # type: ignore
                 slice_z = min_z + (self.diff_z * percent / 100.0)
             else:
-                slice_z = float(args.zslice)  # type: ignore
+                slice_z = float(args.stlread_zslice)  # type: ignore
 
         if slice_z is None:
             slice_z = min_z + (self.diff_z / 2.0)
@@ -57,11 +65,11 @@ class DrawReader(DrawReaderBase):
             last_y = None
             for point in obj:
                 if last_x is not None:
-                    self.add_line((last_x, last_y), (point[0], point[1]))
+                    self._add_line((last_x, last_y), (point[0], point[1]))
                 last_x = point[0]
                 last_y = point[1]
 
-            self.add_line((last_x, last_y), (obj[0][0], obj[0][1]))
+            self._add_line((last_x, last_y), (obj[0][0], obj[0][1]))
 
         self.min_max = [0.0, 0.0, 10.0, 10.0]
         for seg_idx, segment in enumerate(self.segments):
@@ -81,33 +89,6 @@ class DrawReader(DrawReaderBase):
         self.size.append(self.min_max[2] - self.min_max[0])
         self.size.append(self.min_max[3] - self.min_max[1])
 
-    def add_line(self, start, end, layer="0") -> None:
-        dist = round(calc_distance(start, end), 6)
-        if dist > 0.0:
-            self.segments.append(
-                {
-                    "type": "LINE",
-                    "object": None,
-                    "layer": layer,
-                    "start": start,
-                    "end": end,
-                    "bulge": 0.0,
-                }
-            )
-
-    def get_segments(self) -> list[dict]:
-        return self.segments
-
-    def get_minmax(self) -> list[float]:
-        return self.min_max
-
-    def get_size(self) -> list[float]:
-        return self.size
-
-    def draw(self, draw_function, user_data=()) -> None:
-        for segment in self.segments:
-            draw_function(segment["start"], segment["end"], *user_data)
-
     def draw_3d(self):
         GL.glColor4f(1.0, 1.0, 1.0, 0.3)
         GL.glBegin(GL.GL_TRIANGLES)
@@ -120,9 +101,6 @@ class DrawReader(DrawReaderBase):
             GL.glVertex3f(coords[0], coords[1], coords[2] - self.diff_z)
         GL.glEnd()
 
-    def save_tabs(self, tabs: list) -> None:
-        pass
-
     @staticmethod
     def suffix() -> list[str]:
         return ["stl"]
