@@ -4,11 +4,26 @@ import argparse
 
 import freetype
 
-from ..calc import calc_distance, quadratic_bezier  # pylint: disable=E0402
+from ..calc import quadratic_bezier  # pylint: disable=E0402
 from ..input_plugins_base import DrawReaderBase
 
 
 class DrawReader(DrawReaderBase):
+    @staticmethod
+    def arg_parser(parser) -> None:
+        parser.add_argument(
+            "--ttfread-text",
+            help="ttfread: text for the Truetype reader",
+            type=str,
+            default="ViaConstructor",
+        )
+        parser.add_argument(
+            "--ttfread-height",
+            help="text height for the Truetype reader",
+            type=float,
+            default=100,
+        )
+
     def __init__(self, filename: str, args: argparse.Namespace = None):
         """slicing and converting stl into single segments."""
         self.filename = filename
@@ -17,7 +32,7 @@ class DrawReader(DrawReaderBase):
         face = freetype.Face(self.filename)
         face.set_char_size(18 * 64)
 
-        scale = args.text_height / 1000.0  # type: ignore
+        scale = args.ttfread_height / 1000.0  # type: ignore
 
         ctx = {
             "last": (),
@@ -26,7 +41,7 @@ class DrawReader(DrawReaderBase):
             "scale": (scale, scale),
         }
 
-        for char in args.text:  # type: ignore
+        for char in args.ttfread_text:  # type: ignore
             if char == " ":
                 ctx["pos"][0] += 500 * scale  # type: ignore
                 continue
@@ -51,17 +66,21 @@ class DrawReader(DrawReaderBase):
 
         self.min_max = [0.0, 0.0, 10.0, 10.0]
         for seg_idx, segment in enumerate(self.segments):
-            for point in ("start", "end"):
-                if seg_idx == 0:
-                    self.min_max[0] = segment[point][0]
-                    self.min_max[1] = segment[point][1]
-                    self.min_max[2] = segment[point][0]
-                    self.min_max[3] = segment[point][1]
-                else:
-                    self.min_max[0] = min(self.min_max[0], segment[point][0])
-                    self.min_max[1] = min(self.min_max[1], segment[point][1])
-                    self.min_max[2] = max(self.min_max[2], segment[point][0])
-                    self.min_max[3] = max(self.min_max[3], segment[point][1])
+            if seg_idx == 0:
+                self.min_max[0] = segment.start[0]
+                self.min_max[1] = segment.start[1]
+                self.min_max[2] = segment.start[0]
+                self.min_max[3] = segment.start[1]
+            else:
+                self.min_max[0] = min(self.min_max[0], segment.start[0])
+                self.min_max[1] = min(self.min_max[1], segment.start[1])
+                self.min_max[2] = max(self.min_max[2], segment.start[0])
+                self.min_max[3] = max(self.min_max[3], segment.start[1])
+
+                self.min_max[0] = min(self.min_max[0], segment.end[0])
+                self.min_max[1] = min(self.min_max[1], segment.end[1])
+                self.min_max[2] = max(self.min_max[2], segment.end[0])
+                self.min_max[3] = max(self.min_max[3], segment.end[1])
 
         self.size = []
         self.size.append(self.min_max[2] - self.min_max[0])
@@ -81,7 +100,7 @@ class DrawReader(DrawReaderBase):
             point_a.y * ctx["scale"][1] + ctx["pos"][1],
         )
         ctx["max"] = max(ctx["max"], point[0])
-        self.add_line(ctx["last"], point)
+        self._add_line(ctx["last"], point)
         ctx["last"] = point
 
     def conic_to(self, point_a, point_b, ctx):
@@ -103,7 +122,7 @@ class DrawReader(DrawReaderBase):
                 ),
             )
             ctx["max"] = max(ctx["max"], point[0])
-            self.add_line(ctx["last"], point)
+            self._add_line(ctx["last"], point)
             ctx["last"] = point
             curv_pos += 0.1
 
@@ -112,39 +131,6 @@ class DrawReader(DrawReaderBase):
             f"UNSUPPORTED 2nd Cubic Bezier: {point_a.x},{point_a.y} {point_b.x},{point_b.y} {point_c.x},{point_c.y}: {ctx}"
         )
 
-    def add_line(self, start, end, layer="0") -> None:
-        dist = round(calc_distance(start, end), 6)
-        if dist > 0.0:
-            self.segments.append(
-                {
-                    "type": "LINE",
-                    "object": None,
-                    "layer": layer,
-                    "start": start,
-                    "end": end,
-                    "bulge": 0.0,
-                }
-            )
-
-    def get_segments(self) -> list[dict]:
-        return self.segments
-
-    def get_minmax(self) -> list[float]:
-        return self.min_max
-
-    def get_size(self) -> list[float]:
-        return self.size
-
-    def draw(self, draw_function, user_data=()) -> None:
-        for segment in self.segments:
-            draw_function(segment["start"], segment["end"], *user_data)
-
-    def draw_3d(self):
-        pass
-
-    def save_tabs(self, tabs: list) -> None:
-        pass
-
     @staticmethod
     def suffix() -> list[str]:
         return ["ttf"]
