@@ -31,7 +31,12 @@ class PostProcessor:
     def absolute(self, active=True) -> None:
         pass
 
-    def offsets(self, offset="none") -> None:
+    def tool_offsets(self, offset="none") -> None:
+        pass
+
+    def machine_offsets(
+        self, offsets: tuple[float, float, float] = (0.0, 0.0, 0.0)
+    ) -> None:
         pass
 
     def program_start(self) -> None:
@@ -84,9 +89,20 @@ def machine_cmd_begin(project: dict, post: PostProcessor) -> None:
         post.comment(f"Tool-Mode: {project['setup']['maschine']['mode']}")
         post.comment("--------------------------------------------------")
     post.separation()
+
     post.program_start()
+
     post.unit("mm")
-    post.offsets("none")
+    post.tool_offsets("none")
+
+    post.machine_offsets(
+        offsets=(
+            project["setup"]["workpiece"]["offset_x"],
+            project["setup"]["workpiece"]["offset_y"],
+            project["setup"]["workpiece"]["offset_z"],
+        )
+    )
+
     post.absolute(True)
     if project["setup"]["maschine"]["mode"] == "mill" and "Z" in project["axis"]:
         if project["setup"]["mill"]["G64"] > 0.0:
@@ -421,21 +437,28 @@ def segment2machine_cmd(
 
 
 def get_nearest_free_object(
-    polylines, level: int, last_pos: list, milling: set, is_pocket: bool
+    polylines,
+    level: int,
+    last_pos: list,
+    milling: set,
+    is_pocket: bool,
+    next_filter: str,
 ) -> tuple:
     found: bool = False
     nearest_dist: Union[None, float] = None
     nearest_idx: int = 0
     nearest_point = 0
     for offset_num, offset in polylines.items():
-        if (
-            offset_num not in milling
-            and offset.level == level
-            and offset.is_pocket == is_pocket
-            and offset.setup["mill"]["active"]
+        if offset_num not in milling and (  # pylint: disable=R0916
+            (
+                next_filter == ""
+                and offset.level == level
+                and (offset.is_pocket != 0) == is_pocket
+                and offset.setup["mill"]["active"]
+            )
+            or next_filter == offset_num
         ):
-
-            if offset.is_pocket and offset.setup["pockets"]["insideout"]:
+            if offset.is_pocket == 1 and offset.setup["pockets"]["insideout"]:
                 pocket_filter = None
                 offset_num_pre = offset_num.split(".")[0]
                 for pocket_offset_num in polylines:
@@ -484,17 +507,17 @@ def get_nearest_free_object(
                         nearest_idx = offset_num
                         nearest_point = 0
                         found = True
+                    if not offset.fixed_direction:
+                        dist = calc_distance(
+                            last_pos, (vertex_data[0][-1], vertex_data[1][-1])
+                        )
+                        if nearest_dist is None or dist < nearest_dist:
+                            nearest_dist = dist
+                            nearest_idx = offset_num
+                            nearest_point = len(vertex_data[0]) - 1
+                            found = True
 
-                    dist = calc_distance(
-                        last_pos, (vertex_data[0][-1], vertex_data[1][-1])
-                    )
-                    if nearest_dist is None or dist < nearest_dist:
-                        nearest_dist = dist
-                        nearest_idx = offset_num
-                        nearest_point = len(vertex_data[0]) - 1
-                        found = True
-
-    return (found, nearest_idx, nearest_point)
+    return (found, nearest_idx, nearest_point, nearest_dist)
 
 
 def polylines2machine_cmd(project: dict, post: PostProcessor) -> str:
@@ -505,15 +528,28 @@ def polylines2machine_cmd(project: dict, post: PostProcessor) -> str:
     machine_cmd_begin(project, post)
     tabs = project.get("tabs", {})
 
+    next_filter = ""
     order = 0
+    was_pocket = False
     for level in range(project["maxOuter"], -1, -1):
         for is_pocket in (True, False):
             while True:
-                (found, nearest_idx, nearest_point) = get_nearest_free_object(
-                    polylines, level, last_pos, milling, is_pocket
+                (
+                    found,
+                    nearest_idx,
+                    nearest_point,
+                    nearest_dist,
+                ) = get_nearest_free_object(
+                    polylines, level, last_pos, milling, is_pocket, next_filter
                 )
-
+                next_filter = ""
                 if found:
+                    if "." in nearest_idx and nearest_idx.split(".")[-1] == "1":
+                        # get parent after last pocket line
+                        next_filter = f"{nearest_idx.split('.')[0]}.0"
+                        if next_filter in milling:
+                            next_filter = ""
+
                     milling.add(nearest_idx)
                     polyline = polylines[nearest_idx]
                     vertex_data = vertex_data_cache(polyline)
@@ -561,9 +597,9 @@ def polylines2machine_cmd(project: dict, post: PostProcessor) -> str:
                         post.comment(f"Level: {level}")
                         post.comment(f"Order: {order}")
                         post.comment(f"Object: {nearest_idx}")
-                        post.comment(f"Distance: {obj_distance}mm")
+                        post.comment(f"Distance: {round(obj_distance, 4)}mm")
                         post.comment(f"Closed: {is_closed}")
-                        post.comment(f"isPocket: {polyline.is_pocket}")
+                        post.comment(f"isPocket: {polyline.is_pocket != 0}")
                         if (
                             project["setup"]["maschine"]["mode"] != "laser"
                             and "Z" in project["axis"]
@@ -582,21 +618,36 @@ def polylines2machine_cmd(project: dict, post: PostProcessor) -> str:
                             "--------------------------------------------------"
                         )
 
-                    if is_closed:
-                        if (
-                            project["setup"]["maschine"]["mode"] != "laser"
-                            and "Z" in project["axis"]
-                        ):
-                            post.move(z_pos=project["setup"]["mill"]["fast_move_z"])
-                    post.move(x_pos=points[0][0], y_pos=points[0][1])
+                    if polyline.setup["mill"]["step"] >= -0.01:
+                        polyline.setup["mill"]["step"] = -0.01
 
                     depth = polyline.setup["mill"]["step"]
+                    if depth < polyline.setup["mill"]["depth"]:
+                        depth = polyline.setup["mill"]["depth"]
+
+                    if (
+                        project["setup"]["maschine"]["mode"] == "mill"
+                        and "Z" in project["axis"]
+                    ):
+                        if not (
+                            was_pocket
+                            and is_pocket
+                            and nearest_dist < project["setup"]["tool"]["diameter"]
+                        ):
+                            post.move(z_pos=project["setup"]["mill"]["fast_move_z"])
+                        elif helix_mode:
+                            post.move(z_pos=0.0)
+                        else:
+                            post.move(z_pos=depth)
+
                     if (
                         project["setup"]["maschine"]["mode"] != "mill"
                         or "Z" not in project["axis"]
                     ):
                         depth = 0.0
-                        post.spindle_cw(project["setup"]["tool"]["speed"], pause=0)
+                        post.move(z_pos=depth)
+
+                    post.move(x_pos=points[0][0], y_pos=points[0][1])
 
                     last_depth = 0.0
                     while True:
@@ -612,7 +663,7 @@ def polylines2machine_cmd(project: dict, post: PostProcessor) -> str:
 
                         if not is_closed:
                             if (
-                                project["setup"]["maschine"]["mode"] != "laser"
+                                project["setup"]["maschine"]["mode"] == "mill"
                                 and "Z" in project["axis"]
                             ):
                                 post.move(z_pos=project["setup"]["mill"]["fast_move_z"])
@@ -629,6 +680,12 @@ def polylines2machine_cmd(project: dict, post: PostProcessor) -> str:
                                 post.linear(z_pos=depth)
                             post.feedrate(project["setup"]["tool"]["rate_h"])
 
+                        if (
+                            project["setup"]["maschine"]["mode"] != "mill"
+                            or "Z" not in project["axis"]
+                        ):
+                            post.spindle_cw(project["setup"]["tool"]["speed"], pause=0)
+
                         trav_distance = 0
                         last = points[0]
                         for point in points:
@@ -690,8 +747,8 @@ def polylines2machine_cmd(project: dict, post: PostProcessor) -> str:
                         ):
                             break
 
-                    if project["setup"]["maschine"]["mode"] != "laser":
-                        post.move(z_pos=project["setup"]["mill"]["fast_move_z"])
+                    # if project["setup"]["maschine"]["mode"] != "laser":
+                    #    post.move(z_pos=project["setup"]["mill"]["fast_move_z"])
 
                     if project["setup"]["maschine"]["mode"] != "mill":
                         post.spindle_off()
@@ -701,9 +758,10 @@ def polylines2machine_cmd(project: dict, post: PostProcessor) -> str:
                     else:
                         last_pos = points[-1]
                     order += 1
-
                 else:
                     break
 
+                was_pocket = is_pocket
+
     machine_cmd_end(project, post)
     return post.get()
