@@ -121,8 +121,7 @@ def no_translation(text):
 _ = no_translation
 lang = os.environ.get("LANGUAGE")
 if lang:
-    this_dir, this_filename = os.path.split(__file__)
-    localedir = os.path.join(this_dir, "..", "locales")
+    localedir = os.path.join(Path(__file__).resolve().parent, "locales")
     try:
         lang_translations = gettext.translation(
             "base", localedir=localedir, languages=[lang]
@@ -130,7 +129,7 @@ if lang:
         lang_translations.install()
         _ = lang_translations.gettext
     except FileNotFoundError:
-        pass
+        print(f"WARNING: localedir not found {localedir}")
 
 
 class GLWidget(QGLWidget):
@@ -558,6 +557,8 @@ class ViaConstructor:
     status_bar: Optional[QWidget] = None
     main: Optional[QMainWindow] = None
 
+    module_root = Path(__file__).resolve().parent
+
     def save_objects_as_dxf(self, output_file) -> bool:
         try:
             doc = ezdxf.new("R2010")
@@ -607,15 +608,15 @@ class ViaConstructor:
         psetup: dict = self.project["setup"]
         min_max = objects2minmax(self.project["objects"])
         self.project["minMax"] = min_max
-        if psetup["mill"]["zero"] == "bottomLeft":
+        if psetup["workpiece"]["zero"] == "bottomLeft":
             move_objects(self.project["objects"], -min_max[0], -min_max[1])
-        elif psetup["mill"]["zero"] == "bottomRight":
+        elif psetup["workpiece"]["zero"] == "bottomRight":
             move_objects(self.project["objects"], -min_max[2], -min_max[1])
-        elif psetup["mill"]["zero"] == "topLeft":
+        elif psetup["workpiece"]["zero"] == "topLeft":
             move_objects(self.project["objects"], -min_max[0], -min_max[3])
-        elif psetup["mill"]["zero"] == "topRight":
+        elif psetup["workpiece"]["zero"] == "topRight":
             move_objects(self.project["objects"], -min_max[2], -min_max[3])
-        elif psetup["mill"]["zero"] == "center":
+        elif psetup["workpiece"]["zero"] == "center":
             xdiff = min_max[2] - min_max[0]
             ydiff = min_max[3] - min_max[1]
             move_objects(
@@ -1394,12 +1395,7 @@ class ViaConstructor:
         section = ""
 
         for title, toolbutton in toolbuttons.items():
-            icon = os.path.join(self.this_dir, "..", "data", toolbutton[0])
-            if not os.path.isfile(icon):
-                icon = os.path.join("/usr", "data", toolbutton[0])
-            if not os.path.isfile(icon):
-                icon = os.path.join("/usr", "local", "data", toolbutton[0])
-
+            icon = os.path.join(self.module_root, "icons", toolbutton[0])
             if toolbutton[5] != section:
                 self.toolbar.addSeparator()
                 section = toolbutton[5]
@@ -1441,11 +1437,11 @@ class ViaConstructor:
             for ename, entry in self.project["setup_defaults"][sname].items():
                 container = QWidget()
                 hlayout = QHBoxLayout(container)
-                label = QLabel(_(entry.get("title", ename)))
+                label = QLabel(entry.get("title", ename))
                 hlayout.addWidget(label)
                 vlayout.addWidget(container)
                 if entry["type"] == "bool":
-                    checkbox = QCheckBox(_(entry.get("title", ename)))
+                    checkbox = QCheckBox(entry.get("title", ename))
                     checkbox.setChecked(self.project["setup"][sname][ename])
                     checkbox.setToolTip(entry.get("tooltip", f"{sname}/{ename}"))
                     checkbox.stateChanged.connect(self.global_changed)  # type: ignore
@@ -1505,7 +1501,7 @@ class ViaConstructor:
                             button.setIcon(
                                 QIcon(
                                     os.path.join(
-                                        self.this_dir, "..", "data", "select.png"
+                                        self.module_root, "icons", "select.png"
                                     )
                                 )
                             )
